﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing.Design;
using System.Security.Permissions;
using System.IO;
using System.Text.RegularExpressions;
using System.Globalization;
using System.Drawing;

namespace X.FCKeditor
{
    [AspNetHostingPermission(SecurityAction.Demand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [DefaultProperty("Value")]
    [ToolboxData("<{0}:FCKeditor runat=server></{0}:FCKeditor>")]
    [Designer(typeof(FCKeditorDesigner))]
    [ParseChildren(true, "Value")]
    [PersistChildren(false)]
    [DefaultEvent("SaveClick")]
    [ToolboxBitmap(typeof(FCKeditor),"FCKeditor.FCKeditor.ico")]
    public sealed class FCKeditor : Control, IPostBackDataHandler, IPostBackEventHandler, INamingContainer
    {
        #region Appearance
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Localizable(true)]
        public string Value
        {
            get
            {
                String s = (String)ViewState["Value"];
                return ((s == null) ? String.Empty : s);
            }
            set
            {
                ViewState["Value"] = value;
            }
        }

        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue(typeof(Unit), "350px")]
        [Localizable(true)]
        public Unit Height
        {
            get
            {
                Object h = (Object)ViewState["Height"];
                return ((h == null) ? Unit.Pixel(350) : (Unit)h);
            }

            set
            {
                ViewState["Height"] = value;
            }
        }

        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue(typeof(Unit), "100%")]
        [Localizable(true)]
        public Unit Width
        {
            get
            {
                Object w = (Object)ViewState["Width"];
                return ((w == null) ? Unit.Percentage(100) : (Unit)w);
            }
            set
            {
                ViewState["Width"] = value;
            }
        }

        #endregion

        #region FileManager
        [Bindable(true)]
        [Category("FileManager")]
        [DefaultValue(Constants.DEFAULT_UserFilePath)]
        [Localizable(true)]
        public string UserFilePath
        {
            get
            {
                String s = (string)ViewState["UserFilePath"];
                return ((s == null || s == string.Empty) ? Constants.DEFAULT_UserFilePath : s.EndsWith("/") ? s : s + "/");
            }
            set
            {
                ViewState["UserFilePath"] = value;
            }
        }

        [Bindable(true)]
        [Category("FileManager")]
        [DefaultValue(Constants.DEFAULT_UserFileUploaderUrl)]
        [Localizable(true)]
        public string UploaderUrl
        {
            get
            {
                String s = (string)ViewState["UploaderUrl"];
                return ((s == null || s == string.Empty) ? Constants.DEFAULT_UserFileUploaderUrl : s);
            }
            set
            {
                ViewState["UploaderUrl"] = value;
            }
        }

        [Bindable(true)]
        [Category("FileManager")]
        [DefaultValue(Constants.DEFAULT_UserFileBrowserUrl)]
        [Localizable(true)]
        public string BrowserUrl
        {
            get
            {
                String s = (string)ViewState["BrowserUrl"];
                return ((s == null || s == string.Empty) ? Constants.DEFAULT_UserFileBrowserUrl : s);
            }
            set
            {
                ViewState["BrowserUrl"] = value;
            }
        }

        [Bindable(true)]
        [Category("FileManager")]
        [DefaultValue(Constants.DEFAULT_UserFileDeniedExtensions)]
        [Localizable(true)]
        public string UserFileDeniedExtensions
        {
            get
            {
                String s = (string)ViewState["UserFileDeniedExtensions"];
                return ((s == null) ? Constants.DEFAULT_UserFileDeniedExtensions : s.ToLower());
            }
            set
            {
                ViewState["UserFileDeniedExtensions"] = value.ToLower();
            }
        }

        [Bindable(true)]
        [Category("FileManager")]
        [DefaultValue(Constants.DEFAULT_UserFileMaxSize)]
        [Localizable(true)]
        public int UserFileMaxSize
        {
            get
            {
                Object size = (Object)ViewState["UserFileMaxSize"];
                return (size == null || Convert.ToInt32(size) == 0) ? Constants.DEFAULT_UserFileMaxSize : Convert.ToInt32(size);
            }
            set
            {
                ViewState["UserFileMaxSize"] = value;
            }
        }
        [Bindable(true)]
        [Category("FileManager")]
        [DefaultValue(false)]
        [Localizable(true)]
        public bool UserFileCanCreateFolder
        {
            get
            {
                object b = (object)ViewState["UserFileCanCreateFolder"];
                return ((b == null) ? false : (bool)b);
            }
            set
            {
                ViewState["UserFileCanCreateFolder"] = value;
            }
        }
        #endregion

        #region Configuration
        [Bindable(true)]
        [Category("Configuration")]
        [DefaultValue(Constants.DEFAULT_BasePath)]
        [Localizable(true)]
        public string BasePath
        {
            get
            {
                String s = (String)ViewState["BasePath"];
                return ((s == null) ? Constants.DEFAULT_BasePath : s.EndsWith("/") ? s : s + "/");
            }

            set
            {
                ViewState["BasePath"] = value;
            }
        }

        [Bindable(true)]
        [Category("Configuration")]
        [DefaultValue(Constants.DEFAULT_ToolbarSet)]
        [Localizable(true)]
        public string ToolbarSet
        {
            get
            {
                String s = (String)ViewState["ToolbarSet"];
                return ((s == null || s == string.Empty) ?  Constants.DEFAULT_ToolbarSet : s) ;
            }

            set
            {
                ViewState["ToolbarSet"] = value;
            }
        }

        [Bindable(true)]
        [Category("Configuration")]
        [DefaultValue("")]
        [Localizable(true)]
        [Editor(typeof(CustomConfigurationEditor), typeof(UITypeEditor))]
        public string CustomConfiguration
        {
            get
            {
                String s = (String)ViewState["CustomConfiguration"];
                return ((s == null) ? String.Empty : s);
            }

            set
            {
                ViewState["CustomConfiguration"] = value;
            }
        }

        [Bindable(true)]
        [Category("Configuration")]
        [DefaultValue(false)]
        [Localizable(true)]
        public bool OverWriteCustomConfigurationFile
        {
            get
            {
                object b = (object)ViewState["OverWriteCustomConfigurationFile"];
                return ((b == null) ? false : (bool)b);
            }
            set
            {
                ViewState["OverWriteCustomConfigurationFile"] = value;
            }
        }

        #endregion

        #region Render
        string getCustomConfigurationsPath()
        {
            string pagePath = HttpContext.Current.Request.Path;
            string fckcfgVirtualPath = string.Format("{0}.{1}.{2}", pagePath, ClientID, "js");

            string fckcfgPhysicalPath = HttpContext.Current.Server.MapPath(fckcfgVirtualPath);

            if (!File.Exists(fckcfgPhysicalPath) || OverWriteCustomConfigurationFile)
            {
                try
                {
                    FileStream fs = new FileStream(fckcfgPhysicalPath, FileMode.Create);
                    StreamWriter sw = new StreamWriter(fs, Encoding.UTF8);

                    TextWriter tw = StreamWriter.Synchronized(sw);

                    // event support
                    if (saveClick != null)
                    {
                        tw.WriteLine("FCKSaveCommand.prototype.Execute=function(){window.parent." + Page.ClientScript.GetPostBackEventReference(this, string.Empty) + ";};");
                    }
                    else
                    {
                        tw.WriteLine("FCKSaveCommand.prototype.Execute=function(){alert('The SaveEvent is not configured! ');};");
                    }

                    FileManager.Config cfg = new FileManager.Config(UserFilePath, UserFileMaxSize, UserFileDeniedExtensions, UserFileCanCreateFolder);

                    //LinkBrowser & LinkUploader
                    tw.WriteLine(string.Format("FCKConfig.LinkBrowserURL='{0}?Type=File&Config={1}';", ResolveUrl(BrowserUrl), cfg.GetEncodeConfig()));
                    tw.WriteLine(string.Format("FCKConfig.LinkUploadURL='{0}?Type=File&Config={1}';", ResolveUrl(UploaderUrl), cfg.GetEncodeConfig()));

                    //ImageBrowser & ImageUploader
                    tw.WriteLine(string.Format("FCKConfig.ImageBrowserURL='{0}?Type=Image&Config={1}';", ResolveUrl(BrowserUrl), cfg.GetEncodeConfig()));
                    tw.WriteLine(string.Format("FCKConfig.ImageUploadURL='{0}?Type=Image&Config={1}';", ResolveUrl(UploaderUrl), cfg.GetEncodeConfig()));

                    //FlashBrowser & FlashUploader
                    tw.WriteLine(string.Format("FCKConfig.FlashBrowserURL='{0}?Type=Flash&Config={1}';", ResolveUrl(BrowserUrl), cfg.GetEncodeConfig()));
                    tw.WriteLine(string.Format("FCKConfig.FlashUploadURL='{0}?Type=Flash&Config={1}';", ResolveUrl(UploaderUrl), cfg.GetEncodeConfig()));

                    tw.WriteLine(CustomConfiguration);
                    tw.Flush();
                    tw.Close();

                    sw.Close();
                    fs.Close();
                }
                catch
                {
                    //do nothing
                }

            }

            return fckcfgVirtualPath;
        }

        protected override void Render(HtmlTextWriter writer)
        {
            writer.Write("<div>");
            if (checkBrowserCompatibility())
            {
                // Render the linked hidden field.
                writer.Write(string.Format("<input type=\"hidden\" id=\"{0}\" name=\"{1}\" value=\"{2}\" />", ClientID, UniqueID, HttpUtility.HtmlEncode(Value)));

                // Render the configurations hidden field.
                string customConfig = string.Format("CustomConfigurationsPath={0}", getCustomConfigurationsPath());
                writer.Write(string.Format("<input type=\"hidden\" id=\"{0}___Config\" value=\"{1}\" />", ClientID, customConfig));

                // Render the editor IFRAME.
                StringBuilder editorUrlBuilder = new StringBuilder();
                editorUrlBuilder.Append(ResolveClientUrl(BasePath));
                editorUrlBuilder.Append("editor/");
                editorUrlBuilder.Append(Page.Request.QueryString["fcksource"] == "true" ? "fckeditor.original.html" : "fckeditor.html");
                editorUrlBuilder.AppendFormat("?InstanceName={0}",ClientID);
                editorUrlBuilder.AppendFormat("&amp;Toolbar={0}", ToolbarSet);

                writer.Write("<iframe id=\"{0}___Frame\" src=\"{1}\" width=\"{2}\" height=\"{3}\" frameborder=\"no\" scrolling=\"no\"></iframe>", ClientID, editorUrlBuilder.ToString(), Width, Height);
            }
            else
            {
                writer.Write(string.Format("<textarea name=\"{0}\" rows=\"4\" cols=\"40\" style=\"width: {1}; height: {2}\" wrap=\"virtual\">{3}</textarea>", UniqueID, Width, Height, HttpUtility.HtmlEncode(Value)));
            }
            writer.Write("</div>");
        }

        static bool checkBrowserCompatibility()
        {
            System.Web.HttpBrowserCapabilities oBrowser = HttpContext.Current.Request.Browser;

            // Internet Explorer 5.5+ for Windows
            if (oBrowser.Browser == "IE" && (oBrowser.MajorVersion >= 6 || (oBrowser.MajorVersion == 5 && oBrowser.MinorVersion >= 0.5)) && oBrowser.Win32)
                return true;
            else
            {
                Match oMatch = Regex.Match(HttpContext.Current.Request.UserAgent, @"(?<=Gecko/)\d{8}");
                return (oMatch.Success && int.Parse(oMatch.Value, CultureInfo.InvariantCulture) >= 20030210);
            }
        }

        #endregion

        #region IPostBackDataHandler Members

        bool IPostBackDataHandler.LoadPostData(string postDataKey, System.Collections.Specialized.NameValueCollection postCollection)
        {
            string presentValue = Value;
            string postedValue = postCollection[postDataKey];

            if (presentValue == null || !presentValue.Equals(postedValue))
            {
                Value = postedValue;
                return true;
            }

            return false;
        }

        void IPostBackDataHandler.RaisePostDataChangedEvent()
        {
            //do nothing
        }

        #endregion

        #region IPostBackEventHandler Members

        private event EventHandler saveClick;
        [Category("FCKeditor")]
        public event EventHandler SaveClick
        {
            add
            {
                saveClick += value;
            }
            remove
            {
                saveClick -= value;
            }
        }

        void OnSaveClick(EventArgs e)
        {
            if (saveClick != null)
            {
                saveClick(this, e);
            }
        }

        void IPostBackEventHandler.RaisePostBackEvent(string eventArgument)
        {
            OnSaveClick(EventArgs.Empty);
        }

        #endregion
    }
}
