﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Text.RegularExpressions;

namespace X.FCKeditor.FileManager
{
    public partial class FolderLister : System.Web.UI.Page
    {
        string resourceType;
        Config configManager;
        protected void Page_Load(object sender, EventArgs e)
        {
            resourceType = Request["Type"];
            configManager = new Config(Request["Config"]);
            string current;

            if (Request["folder"] == null || Request["folder"] == string.Empty)
            {
                current = configManager.UserFilePath;
                current += resourceType + "/";
            }
            else
            {
                current = Request["folder"];
            }
            loadFolderContents(current);
        }

        void loadFolderContents(string path)
        {
            initListTable();
            loadParentFolder(path);
            loadFolders(path);
            loadFiles(path);
        }

        static TableRow createTableRow(FolderFileInfo data)
        {
            TableRow temp = new TableRow();

            TableCell icon = new TableCell();
            Image imgIcon = new Image();
            imgIcon.Height = imgIcon.Width = Unit.Pixel(16);
            imgIcon.ImageUrl = data.Icon;
            icon.Controls.Add(imgIcon);

            TableCell name = new TableCell();
            name.Text = data.Name;

            TableCell size = new TableCell();
            size.Text = data.Size;
            size.HorizontalAlign = HorizontalAlign.Right;

            TableCell category = new TableCell();
            category.Text = data.Category;

            TableCell modifyTime = new TableCell();
            modifyTime.Text = data.ModifyTime;

            temp.Cells.Add(icon);
            temp.Cells.Add(name);
            temp.Cells.Add(size);
            temp.Cells.Add(category);
            temp.Cells.Add(modifyTime);

            return temp;
        }
        void initListTable()
        {
            while (TableFolderLister.Rows.Count > 1)
            {
                TableFolderLister.Rows.RemoveAt(1);
            }
        }
        void loadParentFolder(string path)
        {
            string targetPath = (path == configManager.UserFilePath) ? path : path.Substring(0, path.TrimEnd("/".ToCharArray()).LastIndexOf("/") + 1);

            FolderFileInfo parentFolder = new FolderFileInfo();
            parentFolder.Icon = GetImage("FolderParent.gif");
            parentFolder.Name = "..";

            TableRow temp = createTableRow(parentFolder);
            temp.Attributes.Add("onmouseover", "this.bgColor='#f7f7f7'");
            temp.Attributes.Add("onmouseout", "this.bgColor='#ffffff'");
            temp.Attributes.Add("onclick", string.Format("showFolder('{0}','{1}');", targetPath, targetPath.Substring(configManager.UserFilePath.Length-1)));

            TableFolderLister.Rows.Add(temp);

        }
        void loadFolders(string path)
        {
            string[] subdirs = Directory.GetDirectories(Server.MapPath(path));
            string targetPath;
            for (int i = 0; i < subdirs.Length; i++)
            {
                DirectoryInfo di=new DirectoryInfo(subdirs[i]);
                FolderFileInfo data = new FolderFileInfo(GetImage("Folder.gif"), di);
                TableRow temp = createTableRow(data);
                targetPath = path + di.Name + "/";
                temp.Attributes.Add("onmouseover", "this.bgColor='#f7f7f7'");
                temp.Attributes.Add("onmouseout", "this.bgColor='#ffffff'");
                temp.Attributes.Add("onclick", string.Format("showFolder('{0}','{1}');", targetPath, targetPath.Substring(configManager.UserFilePath.Length-1)));


                TableFolderLister.Rows.Add(temp);
            }
        }

        void loadFiles(string path)
        {
            string[] files = Directory.GetFiles(Server.MapPath(path));
            for (int i = 0; i < files.Length; i++)
            {
                FileInfo fi = new FileInfo(files[i]);
                string ext = fi.Extension.ToLower();
                string icon = Regex.IsMatch(ext, Constants.AvailableIconExts) ? GetIcon(ext.Substring(1) + ".gif") : GetIcon("default.gif");
                FolderFileInfo data = new FolderFileInfo(icon, fi);

                TableRow temp = createTableRow(data);
                string targetPath = path + fi.Name;
                if (data.Category == "Image")
                {
                    temp.Attributes.Add("onmouseover", string.Format("this.bgColor='#f7f7f7';previewImage(event,'{0}',true);", HttpUtility.UrlDecode(ResolveUrl(targetPath))));
                    temp.Attributes.Add("onmouseout", string.Format("this.bgColor='#ffffff';previewImage(event,'{0}',false);", HttpUtility.UrlDecode(ResolveUrl(targetPath))));
                }
                else
                {
                    temp.Attributes.Add("onmouseover", "this.bgColor='#f7f7f7'");
                    temp.Attributes.Add("onmouseout", "this.bgColor='#ffffff'");
                }
                temp.Attributes.Add("onclick", string.Format("selectFile('{0}');", HttpUtility.UrlDecode(ResolveUrl(targetPath))));

                TableFolderLister.Rows.Add(temp);
            }
        }

        public string GetImage(string name)
        {
            return ClientScript.GetWebResourceUrl(typeof(FCKeditor), "X.FCKeditor.FileManager.images." + name);
        }

        public string GetIcon(string name)
        {
            return ClientScript.GetWebResourceUrl(typeof(FCKeditor), "X.FCKeditor.FileManager.images.ext." + name);
        }
    }
}
