﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text.RegularExpressions;
using System.IO;

namespace X.FCKeditor.FileManager
{
    public partial class Uploader : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            string resourceType = Request["Type"];
            Config configManager = new Config(Request["Config"]);
            try
            {
                HttpPostedFile newFile = Request.Files["NewFile"];
                string saveFolder;

                if (Request["saveFolder"] == null || Request["saveFolder"] == string.Empty)
                {
                    switch (resourceType)
                    {
                        case "File":
                            saveFolder = "/File/";
                            break;
                        case "Image":
                            saveFolder = "/Image/";
                            break;
                        case "Flash":
                            saveFolder = "/Flash/";
                            break;
                        default:
                            saveFolder = "/";
                            break;
                    }
                }
                else
                {
                    saveFolder = Request["saveFolder"];
                }

                string saveVirtualFolder = configManager.UserFilePath.TrimEnd("/".ToCharArray()) + saveFolder;

                string savePhysicalFolder = Server.MapPath(saveVirtualFolder);
                string deniedExts = configManager.UserFileDeniedExtensions;
                int maxFileSize = configManager.UserFileMaxSize;

                if (newFile == null || newFile.ContentLength == 0)
                {
                    sendUploadResults(ErrorNumber.FileInvalid, "上传的文件不能为空。");
                    return;
                }

                if (newFile.ContentLength > maxFileSize * 1024)
                {
                    sendUploadResults(ErrorNumber.CustomError, "上传的文件不能超过 " + maxFileSize + " KB。");
                    return;
                }

                if (Regex.IsMatch(new FileInfo(newFile.FileName).Extension.ToLower(), deniedExts))
                {
                    sendUploadResults(ErrorNumber.CustomError, "不允许上传该类型的文件。");
                    return;
                }

                //SaveFile

                string fileName = Path.GetFileName(newFile.FileName);
                string savedFileName;
                string fileUrl = string.Empty;
                int count = 0;

                ErrorNumber err = ErrorNumber.Success;
                string errMessage = string.Empty;

                while (true)
                {
                    savedFileName = Path.Combine(savePhysicalFolder, fileName);
                    if (File.Exists(savedFileName))
                    {
                        count++;
                        fileName = string.Format("{0}({1}){2}", Path.GetFileNameWithoutExtension(newFile.FileName), count, Path.GetExtension(newFile.FileName));
                        err = ErrorNumber.FileExisted;
                        errMessage = string.Format("文件重名。上传的文件被重命名为 “{0}” 。", fileName);
                    }
                    else
                    {
                        newFile.SaveAs(savedFileName);
                        fileUrl = ResolveClientUrl(ResolveUrl((saveVirtualFolder + fileName)));
                        break;
                    }
                }

                sendUploadResults(err, fileUrl, fileName, errMessage);
            }
            catch (Exception ex)
            {
                sendUploadResults(ErrorNumber.CustomError, ex.Message);
            }
        }

        enum ErrorNumber
        {
            Success = 0,
            CustomError = 1,
            FileExisted = 201,
            FileInvalid = 202,
            SecurityError = 203
        }
        void sendUploadResults(ErrorNumber errorNumber, string errorMessage)
        {
            sendUploadResults(errorNumber, string.Empty, string.Empty, errorMessage);
        }

        void sendUploadResults(ErrorNumber errorNumber, string fileUrl, string fileName, string errorMessage)
        {
            Response.Clear();

            Response.Write("<script type='text/javascript' language='javascript'>");
            Response.Write(string.Format("window.parent.OnUploadCompleted({0},'{1}','{2}','{3}');", new object[] { (int)errorNumber, fileUrl.Replace("'", "\\'"), fileName.Replace("'", "\\'"), errorMessage.Replace("'", "\\'") }));
            Response.Write("</script>");
            Response.Flush();
            Context.ApplicationInstance.CompleteRequest();
        }
    }
}
